// ==================================================
// Copyright 2013.
// Siemens Product Lifecycle Management Software Inc.
// All Rights Reserved.
// ==================================================

#ifndef TEAMCENTER_BASE_UTILS_ERRORSTOREPROTECTOR_HXX
#define TEAMCENTER_BASE_UTILS_ERRORSTOREPROTECTOR_HXX

#include <unidefs.h>

#include <base_utils/libbase_utils_exports.h>

/**
    @brief A convenient wrapper for setting/clearing a protect mark on the error stack.

    In Teamcenter, all errors are posted to a single central stack of errors.
    <br> An error on the stack consists of an integer error code, a severity and zero or
    more strings that go into the error message.

    This class is a  convenient wrapper for setting/clearing a protect mark on the error
    stack, useful for when one wants to assert that there is other processing
    and any errors that might occur shall be ignored.

    @note
    The following points needs to be considered whether to return OK rather than
    NOT_OK when within the scope of an ErrorStoreProtector:

    - if the error store was empty before the protector is constructed,
      then almost certainly OK shall be returned, so that the caller doesn't see the invalid
      combination of NOT_OK with an empty stack store

    - if, on the other hand, it is used in some post-error tidy-up and to avoid
      masking the interesting errors, then almost certainly NOT_OK shall be returned.

    @code
    //{ } scope is added intentionally to clear from error stack the errors
    //added by the functions called within this cope.
    {
        ErrorStoreProtector ignoreErrors;

        Function1();
        Function2();
    }
    @endcode
*/
class BASE_UTILS_API ErrorStoreProtector
{
public:

    /**
        Constructor
        <br> The constructor function sets a protect mark on the error stack.
    */
    ErrorStoreProtector();

    /**
        Default destructor.
        <br> The constructor function clears the error stack from the protect mark.
    */
    virtual ~ErrorStoreProtector();

    /**
        Clears a protect mark on the error stack.
    */
    void clear();

    /**
        Forgets the protect mark and error stack will not be cleared.
    */
    void forget();

    /**
        Remembers the protect mark and error stack will be cleared.
    */
    void remember();

private:

    /**
        Private default constructor
    */
    ErrorStoreProtector( const ErrorStoreProtector&  other );

    /**
        Copy constructor
    */
    ErrorStoreProtector&  operator=( const ErrorStoreProtector&  other );

    /**
        The protect mark
    */
    int m_errorStoreMarkpoint;

    bool m_forgotten;
    bool m_cleared;
};

#include <base_utils/libbase_utils_undef.h>
#endif
